# -*- coding: utf-8
# $Id: FLE.py,v 1.4 2003/08/18 07:44:24 berlingo Exp $
#
# Copyright 2001, 2002 by IVA Team and contributors
#
# This file is part of IVA.
#
# IVA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# IVA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with IVA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""This is the main module for FLE, containing the FLE class and the Zope factory method for creating a FLE installation inside Zope."""

__version__ = "$Revision: 1.4 $"[11:-2]
IVA_VERSION = "IVA 0.6.1-upgrader"

import os.path, string, time

try:
    from PIL import Image
    PIL_imported = 1
except ImportError:
    PIL_imported = 0

import Globals
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import Persistent, Acquisition, HTMLFile, PersistentMapping
import AccessControl
import OFS
from OFS.Folder import Folder
from Products.ZCatalog.ZCatalog import ZCatalog

from TraversableWrapper import TraversableWrapper as TW
from Cruft import Cruft

from ThinkingTypeSetManager import ThinkingTypeSetManager as TTSM
from UserManager import UserManager
from CourseManager import CourseManager
from Timer import Timer
from common import file_path, ui_path, translate
import Acquisition
import YlTest
from string import atoi

from common import add_dtml_obj, new_add_dtml_obj, add_image_obj, \
     reload_dtml, add_dtml, reload_pt, add_pt_obj, add_pt
from common import perm_view, perm_edit, perm_manage, perm_add_lo, perm_access, \
     roles_admin, roles_staff, roles_user
import common
import re

import YlHoidla
from YlHoidla import YlHoidla

try:
	from Products.CMFCore.CookieCrumbler import CookieCrumbler, ATTEMPT_NONE, ATTEMPT_LOGIN, ATTEMPT_RESUME
	CCAware = 1
except:
	CCAware = 0

# This is the class for the FLE installation root object.
class FLE(
    Folder,
    TW,
    Cruft,
    Persistent,
    AccessControl.Role.RoleManager,
    OFS.SimpleItem.Item,
    Timer,
    ):
    """FLE product."""
    security = AccessControl.ClassSecurityInfo()
    security.declareObjectPublic()

    meta_type = 'IVA'

    security.declareProtected(perm_manage, 'management_tab')
#    management_tab = Globals.DTMLFile('ui/FLE/management_tab', globals())
    management_tab = PageTemplateFile('ui/management_tab', globals())
    management_tab._owner = None
    manage_options=(
        {'label' : 'Tools', 'action' : 'management_tab'},
        ) + OFS.Folder.Folder.manage_options

    dtml_files = (
	('style_css','','ui/style_css'),
	('style_cssVeebilaud','','ui/style_cssVeebilaud'),
	('style_cssRaamaturiiul','','ui/style_cssRaamaturiiul'),
	('style_cssTootuba','','ui/style_cssTootuba'),
	('style_cssHaldus','','ui/style_cssHaldus'),
	('style_cssOrganizer','','ui/style_cssOrganizer'),
	('js/checkAll2','','ui/js/checkAll2'),
	('js/tree','','ui/js/tree'),
	('js/ivaeditor', '', 'ui/js/ivaeditor'),
    	)
    #index_html = Globals.HTMLFile("index_html", globals())
    pt_files = ( # ZWiki files
    	('courses_list','list of courses','ui/courses_list'),
	('password_form','','ui/password_form'),
	('backlinks','ZWiki backlinks','ui/Wiki/backlinks.pt'),
	('badtemplate','ZWiki ZPT','ui/Wiki/badtemplate.pt'),
	('contentspage','ZWiki','ui/Wiki/contentspage.pt'),
	('diffform','ZWiki','ui/Wiki/diffform.pt'),
	('editform','ZWiki','ui/Wiki/editform.pt'),
	('filterissues','ZWiki','ui/Wiki/filterissues.pt'),
	('issuetracker','ZWiki ZPT','ui/Wiki/issuetracker.pt'),
	('ratingform','ZWiki ZPT','ui/Wiki/ratingform.pt'),
	('recentchanges','ZWiki','ui/Wiki/recentchanges.pt'),
	('searchwiki','ZWiki','ui/Wiki/searchwiki.pt'),
	('subscribeform','ZWiki','ui/Wiki/subscribeform.pt'),
	('useroptions','ZWiki','ui/Wiki/useroptions.pt'),
	('wikipage_macros','ZWiki','ui/Wiki/wikipage_macros.pt'),
	('wikipage','ZWiki Page','ui/Wiki/wikipage.pt'),

	('index_html','start page','ui/index_html'),
	('start_page','start page','ui/start_page'),
    	('defaultRender','dummy page to display custom pages','ui/defaultRender'),
	('changeEvent','','ui/changeEvent'),
	('createMail','','ui/createMail'),
	('event_index','','ui/event_index'),
	('event_list','','ui/event_list'),
	('haldusleht','','ui/haldusleht'),
	('kontrolli_kursuse_kaust','','ui/kontrolli_kursuse_kaust'),
	('mailbox_index','','ui/mailbox_index'),
	('mail_index','','ui/mail_index'),
	('mailStatus','','ui/mailStatus'),
	('main_macro','','ui/main_macro'),
	('manage_addAssignment','','ui/manage_addAssignment'),
	('manage_addingUsers','','ui/manage_addingUsers'),
	('manage_addIvaUsers','','ui/manage_addIvaUsers'),
	('manage_affiliations','','ui/manage_affiliations'),
	('manage_assignments','','ui/manage_assignments'),
	('manage_changeAssignment','','ui/manage_changeAssignment'),
	('manage_course_info','','ui/manage_course_info'),
	('manage_gradeAssignment','','ui/manage_gradeAssignment'),
	('manage_iva_setup','','ui/manage_iva_setup'),
	('manage_ivaUsers','','ui/manage_ivaUsers'),
	('manage_participants','','ui/manage_participants'),
	('manage_userPermissions','','ui/manage_userPermissions'),
	('manage_pendingUsers','','ui/manage_pendingUsers'),
	('message_dialog','','ui/message_dialog'),
	('message_dialog2','','ui/message_dialog2'),
	('message_dialog_error','','ui/message_dialog_error'),
	('organizer_calendar','','ui/organizer_calendar'),
	('organizer_change_course','','ui/organizer_change_course'),
	('organizer_index','','ui/organizer_index'),
	('page_macro','','ui/page_macro'),
	('sea_jooksev_kursus','','ui/sea_jooksev_kursus'),
	('sharedDocs','','ui/sharedDocs'),
	('standard_error_message','','ui/standard_error_message'),
	('assignmentGrading','','ui/assignmentGrading'),
	('testihaldus', '', 'ui/testihaldus'),
	('tootubade_leht','','ui/tootubade_leht'),
	('workshop_subgroup','','ui/workshop_subgroup'),
	('workshop_subgroup_add','','ui/workshop_subgroup_add'),
	('workshop_subgroup_list','','ui/workshop_subgroup_list'),
	('workshop_subgroup_show','','ui/workshop_subgroup_show'),
	('wt_add_folder','','ui/wt_add_folder'),
	('wt_add_link','','ui/wt_add_link'),
	('wt_add_memo','','ui/wt_add_memo'),
	('wt_add_wiki','','ui/wt_add_wiki'),
	('wt_index_html','','ui/wt_index_html'),
	('wt_itemProperties','','ui/wt_itemProperties'),
	('wt_manage','','ui/wt_manage'),
	('wt_rename','','ui/wt_rename'),
	('wt_upload','','ui/wt_upload'),
	('wt_usercomments','','ui/wt_usercomments'),
	('wt_userForm','','ui/wt_userForm'),
	('wt_view_memo','','ui/wt_view_memo'),
	('wt_zip_upload','','ui/wt_zip_upload'),
	('ylAlus_muutmisVorm', '', 'ui/ylAlus_muutmisVorm'),
	('ylMeta_vorm1', '', 'ui/ylMeta_vorm1'),
	('ylesandedMuutmisTabelina', '', 'ui/ylesandedMuutmisTabelina'),
	('ylOtsing', '', 'ui/ylOtsing'),
	('ylOtsinguVorm', '', 'ui/ylOtsinguVorm'),
	('ylTestSettingsForm', '', 'ui/ylTestSettingsForm'),
	('ylBlankQuestion', '', 'ui/ylBlankQuestion'),
	('ylAddPicture', '', 'ui/ylAddPicture'),
	('ylTable', '', 'ui/ylTable'),
	('qtChangeFormMatchingExercise', '', 'ui/qtChangeFormMatchingExercise'),
	('qtChangeFormNumberSpaceExercise', '', 'ui/qtChangeFormNumberSpaceExercise'),
	('qtChangeFormShortAnswerExercise', '', 'ui/qtChangeFormShortAnswerExercise'),
	('qtChangeFormPercentExercise', '', 'ui/qtChangeFormPercentExercise'),
	('qtTimeSettings', '', 'ui/qtTimeSettings'),
	('qtTestAnswerTable', '', 'ui/qtTestAnswerTable'),
	('qtTestAnswerCommentingTable', '', 'ui/qtTestAnswerCommentingTable'),
	('qtTestAnswerStatistics', '', 'ui/qtTestAnswerStatistics'),
	('qtUserResults', '', 'ui/qtUserResults'),
	('qtExplanationQuiz', '', 'ui/qtExplanationQuiz'),
	('qtAnsweringForm', '', 'ui/qtAnsweringForm'),
	('qtAnswersSaved', '', 'ui/qtAnswersSaved'),
	('qtExerciseForm', '', 'ui/qtExerciseForm'),
	('qtExerciseAnswer', '', 'ui/qtExerciseAnswer'),

	('courses/add_course_context_form','Add Course Context','ui/courses/add_course_context_form'),
	('courses/context_html','Context index','ui/courses/context_html'),
	('courses/course_html','Course Context list','ui/courses/course_html'),
	('courses/course_info','','ui/courses/course_info'),
	('courses/course_setup_roleplay_form','Roleplay setup','ui/courses/course_setup_roleplay_form'),
	('courses/note_edit_form','Note editing form','ui/courses/note_edit_form'),
	('courses/index_html','','ui/courses/index_html'),
	('courses/note_preview_form','Note editing form','ui/courses/note_preview_form'),
	('courses/note_index_html','Note viewing form','ui/courses/note_index_html'),
	('courses/organizer_openCourses','','ui/courses/organizer_openCourses'),
	('courses/printer_macro','Note printers','ui/courses/printer_macro'),
	('courses/note_hide','Note printers','ui/courses/note_hide'),

	('courses/jamming_index_html','Index page of jamming','ui/courses/jamming_index_html'),
	('courses/jam_add_session_form','add jam session','ui/courses/jam_add_session_form'),
	('courses/jamsession_index_html','Index page of jamsession','ui/courses/jamsession_index_html'),
	('courses/jam_add_artefact_form','Add artefact form','ui/courses/jam_add_artefact_form'),
	('courses/jam_artefact_index_html','Artefact index_html','ui/courses/jam_artefact_index_html'),
	('courses/jam_add_annotation_form','Add artefact annotation form','ui/courses/jam_add_annotation_form'),
	('courses/list_readers','Who has read it!?','ui/courses/list_readers'),

	('courses/manage_import_form','Import course','ui/courses/manage_import_form'),
	('courses/teacher_import_form','Import course(for teacher)','ui/courses/teacher_import_form'),
	('courses/manageCategories','manage course categories','ui/courses/manageCategories'),
	('courses/course_index','Welcome page for course','ui/courses/course_index'),
	('courses/courseStatistics','Course statistics','ui/courses/courseStatistics'),
	('courses/describeTTS','Describes thinkingtypeset','ui/courses/describeTTS'),

	('typesets/edit_form_1_3','','ui/typesets/edit_form_1_3'),
	('typesets/edit_form_2_3','','ui/typesets/edit_form_2_3'),
	('typesets/edit_form_3_3','','ui/typesets/edit_form_3_3'),
	('typesets/index_html','','ui/typesets/index_html'),
	('typesets/tts_index','','ui/typesets/tts_index'),
	('typesets/manage_import_form','Import typesets','ui/typesets/manage_import_form'),

	('register_form','Register form','ui/register_form'),

	('fle_users/course_info_user','Information about user actions','ui/fle_users/course_info_user'),
	('fle_users/user_courses_info','Information about all user courses','ui/fle_users/user_courses_info'),
	('fle_users/show_user_info','','ui/fle_users/show_user_info'),
	('fle_users/edit_user_form','','ui/fle_users/edit_user_form'),

	)

    # NOTE: Since we use a python method index_html to do
    # the redirection to the appropriate DTML method, EVERY FOLDER
    # that the user access must have this index_html method
    # (either through class inheritance or as its own dtml method,
    # but not through acquisition from its containers). Acquisition
    # cannot be used, since the object reference passed to the DTML
    # method will point to the object where index_html was acquired.
    # So all courses, contexts, notes, userinfos, webtops and webtopfolders
    # and any other folderish objects must have this python method.
    #security.declarePublic('index_html')
    #def index_html(self,REQUEST):
    #    return self.ui.FLE.index_html(self,REQUEST)

    # Parameters are received from the creation form at
    # ui/FLE/creation_form.dtml
    def __init__(
        self, _id, title,
        fle_admin, fle_admin_pwd,
        fle_admin_first_name, fle_admin_last_name,
        acl_users_mode, smtp_host, smtp_port=25, import_data=None,
        _default_lang='en', _quota=3000000, mfrom='iva@htk.tpu.ee',
	def_passwd=0,def_not_change=0,def_not_aff=0):
        """Construct FLE object."""

        self.setup_timer()

        errors = []
        if not _id:
            errors.append('ID not specified')
        if not title:
            errors.append('Title not specified')
#        if not fle_admin:
#            errors.append('Administrator user name not specified')
        if not fle_admin_pwd and acl_users_mode=='create':
            errors.append('Administrator password not specified')
        if errors:
            raise 'Insufficient form data',str(errors)

        self.id = _id
        self._default_lang=_default_lang
        self._quota=_quota
        self.title = title
	self.mfrom = mfrom
	
	self.def_asswd= int(def_passwd)
	self.def_ot_change= int(def_not_change)
	self.def_ot_aff= int(def_not_aff)
	self._pending_users = PersistentMapping() # user list 

        self.__fle_root = ''

        # The values are saved so that they can be used in
        # manage_afterAdd (where they are also deleted...)
        tmptbl = {}
        tmptbl['initial_fle_admin'] = fle_admin
        tmptbl['initial_fle_admin_pwd'] = fle_admin_pwd
        tmptbl['initial_fle_admin_first_name'] = fle_admin_first_name
        tmptbl['initial_fle_admin_last_name'] = fle_admin_last_name
        tmptbl['acl_users_mode'] = (acl_users_mode=='create')
        # Check that the fileupload exists and has a filename
        if import_data and import_data.filename:
            tmptbl['import_data'] = import_data
        else:
            tmptbl['import_data'] = None
            # If not, set it to none, so we only need to check one
            # thing if we need to decide whether we have a file or not.
            import_data = None

        self.tmptbl = tmptbl

        if len(smtp_host)>0 and smtp_port:
            from Products.MailHost import MailHost
            mailhost = MailHost.MailHost()
            mailhost._init(smtp_host, atoi(smtp_port))
            self._setObject('MailHost', mailhost)

        # Style sheets
#        self.reload_style_sheets()

        # Add images to images/
        self.manage_addFolder('images', '')
        from common import image_file_path
        self.add_images(self.images, image_file_path)

        # ZCatalog for webtop items
        catalog = ZCatalog('catalog_webtop_items', 'ZCatalog for webtop items')

        # indexes
        catalog.addIndex('get_name', 'TextIndex')
        catalog.addIndex('get_content', 'TextIndex')
        catalog.addIndex('get_author', 'FieldIndex')

        # metadata
        catalog.addColumn('get_name')
        catalog.addColumn('get_author')
        catalog.addColumn('absolute_url')

        self._setObject('catalog_webtop_items', catalog)

    security.declarePrivate('manage_afterAdd')
    # Executes a full FLE import if the export data was
    # passed to the constructor (ie. uploaded via the creation
    # form.
    def manage_afterAdd(self, item, container):
        """Add roles to the root level Fle folder and do other
        initialization tasks."""
	
	if not hasattr(self.aq_self, 'qtCatalog'):
		self._setObject('qtCatalog', YlHoidla())

        if hasattr(self,'typesets'):
            # If the typesets directory already exists, then this
            # isn't the initial installation, but rather a rename
            # or a move operation.
            # Skip everything!
            return

        t = self.tmptbl

        # Property telling whether or not we're using an external
        # user folder.
        # Ramifications:
        # - new users are allowed to login
        # - the user folder is read-only
        self.manage_addProperty('allow_external_users', not t['acl_users_mode'], 'boolean')
        # add properties for todos
        self.manage_addProperty('todo_mode', 0, 'boolean')
        self.manage_addProperty('todo_server', '', 'string')
        self.manage_addProperty('todo_login', '', 'string')
        self.manage_addProperty('todo_password', '', 'string')
        self.manage_addProperty('Autoregister', 0, 'boolean')
        # add property for developer controls
        self.manage_addProperty('show_developer_controls', 0, 'boolean')
        # add property for FLE version number
        
        self.manage_addProperty('webtop_quota', 1, 'boolean')
        self.manage_addProperty('webtop_quota_amount', self._quota, 'string')
        self.manage_addProperty('default_language', self._default_lang, 'string')
                
        self.manage_addProperty('IVA_VERSION', IVA_VERSION, 'string')
	self.manage_addProperty('MFROM',self.mfrom,'string')
	self.manage_addProperty('def_passwd',int(self.def_asswd),'boolean')
	self.manage_addProperty('def_not_change',int(self.def_ot_change),'boolean')
	self.manage_addProperty('def_not_aff',int(self.def_ot_aff),'boolean')

        tts_manager = TTSM('typesets', '')
        self._setObject('typesets', tts_manager)
        tts_manager=tts_manager.__of__(self)

        from Downloader import Downloader
        self._setObject('download',Downloader())

	self.manage_addFolder('uploads','Wiki uploads')
	self.manage_addFolder('js','JavaScripts')

        from common import fle_roles

        for role in fle_roles:
            self._addRole(role)
	self._addRole('Items')

	self.manage_permission(perm_access, ('Manager','IVAAdmin','Member','Owner') ,0)
	self.manage_permission(perm_view, ('Authenticated','Manager','IVAAdmin','Member','Owner') ,0)
	self.manage_permission(perm_edit, ('Manager',), 0)

        self.manage_permission(perm_manage, roles_admin, 0)
        self.manage_permission(perm_add_lo, roles_admin, 0)
	try:
		self.manage_permission('Zwiki: Add comments', roles_user+('Authenticated','Owner',), 0)
		self.manage_permission('Zwiki: Add pages', ('Owner',), 0)
		self.manage_permission('Zwiki: Add wikis', ('Owner',), 0)
		self.manage_permission('Zwiki: Change page types', ('Owner',), 0)
		self.manage_permission('Zwiki: Delete pages', ('Owner',), 0)
		self.manage_permission('Zwiki: Edit pages', ('Owner',), 0)
		self.manage_permission('Zwiki: Rename pages', ('Owner',), 0)
		self.manage_permission('Zwiki: Reparent pages', ('Owner',), 0)
	except ValueError:
		pass
	self.uploads.manage_permission('Add Documents, Images, and Files', roles_user+('Authenticated','Owner',), 0)

        um_obj = UserManager('fle_users',
                             #'FLE User Manager')
                             '')
        self._setObject('fle_users',
                        um_obj)

        import_data = t['import_data']

        # Create default knowledge type sets, whether importing
        # or not.
        tts_manager.load_default_sets()

        # Let's do courses last, since if we're importing, we
        # need to have the users already in place.
        cm_obj = CourseManager('courses')
        self._setObject('courses', cm_obj)
        
        self.manage_addProduct['OFSP'].manage_addFolder('css')
        #self.css._setObject('style_css', self.style_css)

        # We don't need these any more.
        del self.tmptbl

	self.reload_pt()
        # Set the zope manageable dtml objects of this class.
        self.reload_dtml()

	self.setDefaultPermissions()
	#XXX: migrate me too!?
	if CCAware:
		self.addCookieCrumbler()

    security.declareProtected(perm_manage, 'reload_pt')
    def reload_pt(self, REQUEST=None):
	""" reload ZopePageTemplates """
	reload_pt(self, self.pt_files)
	self.setDefaultPermissions()

        if REQUEST:
            return self.message_dialog(
                self, REQUEST,
                title="Reloaded",
                message="All page templates reloaded",
                action='index_html')
		
    security.declarePublic('setDefaultPermissions')
    def setDefaultPermissions(self):
	""" doc"""
        getattr(self.images,'iva_logo_trans').manage_permission('View',['Authenticated', 'Anonymous',], 1)
        getattr(self.images,'start_top_bg').manage_permission('View',['Authenticated', 'Anonymous',], 1)
        getattr(self.images,'start_bg').manage_permission('View',['Authenticated', 'Anonymous',], 1)
        getattr(self,'register_form').manage_permission('View',['Authenticated', 'Anonymous',], 1)
        getattr(self,'courses_list').manage_permission('View',['Authenticated', 'Anonymous',], 1)
	try:
	        getattr(self,'style_css').manage_permission('View',['Authenticated', 'Anonymous',], 1)
	except:
		pass
        getattr(self,'index_html').manage_permission('View',['Authenticated', 'Anonymous',], 1)
        getattr(self,'start_page').manage_permission('View',['Authenticated', 'Anonymous',], 1)
	getattr(self,'password_form').manage_permission(perm_view,['Authenticated', 'Anonymous',], 1)
#	getattr(self,'style_css').manage_permission(perm_view,['Anonymous'],0)
	getattr(self,'register_form').manage_permission(perm_view,['Anonymous'],0)
	getattr(self,'images').manage_permission('View',['Anonymous',],1)
	try:
		getattr(getattr(self,'js'), 'tree').manage_permission('View',['Anonymous',],1)
	except AttributeError:
		pass
	getattr(self, 'main_macro').manage_permission('View',['Anonymous',],1)

    security.declareProtected(perm_manage, 'reload_dtml')
    def reload_dtml(self, REQUEST=None):
        """Reloads dtml stuff."""
        reload_dtml(self, self.dtml_files)
	self.setDefaultPermissions()

        if REQUEST:
            return self.message_dialog(
                self, REQUEST,
                title='Reloaded',
                message='dtml files reloaded',
                action='index_html')

    security.declareProtected(perm_manage, 'reload_typesets')
    def reload_typesets(self,REQUEST=None):
        """Reload typesets."""
        self.typesets.load_default_sets()
        if REQUEST:
            return self.message_dialog(
                self, REQUEST,
                title='Reloaded',
                message='Knowledge type sets reloaded',
                action='index_html')


    security.declareProtected(perm_manage, 'reload_images')
    def reload_images(self,REQUEST=None):
        """Reload images."""
        for e in self.images.objectIds():
            self.images.manage_delObjects(e)

        from common import image_file_path
        self.add_images(self.images, image_file_path)

        if REQUEST:
            return self.message_dialog(
                self, REQUEST,
                title='Reloaded',
                message='Images reloaded',
                action='index_html')

    security.declarePrivate('add_images')
    # fs_path is CVS path
    def add_images(self, zope_path, fs_path):
        """Add images recursively."""
        for e in os.listdir(fs_path):
            full_path = os.path.join(fs_path, e)

            if os.path.isdir(full_path):
                if e == 'CVS': continue
		if e == '.svn': continue
                zope_path.manage_addFolder(e, '')
                self.add_images(getattr(zope_path, e), full_path)
            else:
                # title (3rd parameter) will be used as an alt parameter
                # inside HTML <img> tag. (Override in DTML code when you
                # want some alt text (i.e. almost always...))

                # This can be like this for now .... -granbo
                import re
                m = re.match("^(.*)\.", e)

                add_image_obj(zope_path, m.group(1), '', full_path)

    security.declarePrivate('add_style_sheets')
    def add_style_sheets(self, fs_path):
        """Add style sheets."""

        for e in os.listdir(fs_path):             # fetch all style sheets
            if e[-4:] != 'dtml':
                continue
            if e == 'personal_style_sheet.dtml':  # This goes under UserInfo
                continue                          # object (see UserInfo.py).
            full_path = os.path.join(fs_path, e)
            if os.path.isdir(full_path):
                continue                          # skip the CVS directory
            # get the  code
            f = open(full_path)
            code = f.read()
            f.close()


    security.declareProtected(perm_manage, 'reload_all')
    def reload_all(self, REQUEST=None):
        """Reload all dtml, style sheet, image, printer, and
        translation files."""
	self.IVA_VERSION = IVA_VERSION
#        self.reload_all_dtml()
        self.reload_images()
	self.reload_pt()
        self.reload_typesets()
	self.reload_all_dtml()

        if REQUEST:
            return self.message_dialog(
                self, REQUEST,
                title='Reloaded',
                message='Reloaded all dtml, css, images, and knowledge type sets',
                action='index_html')

    security.declareProtected(perm_manage, 'reload_all_dtml')
    def reload_all_dtml(self, REQUEST=None):
        """Reload _all_ dtml files."""
        from common import tree_reload_dtml
        reload_dtml(self, self.dtml_files)
	self.setDefaultPermissions()

        if REQUEST:
            return self.message_dialog(
                self, REQUEST,
                title='Reloaded',
                message='dtml files reloaded',
                action='index_html')
                
#    security.declareProtected(perm_view, 'get_version')
    security.declarePublic('get_version')
    def get_version(self):
	""" return IVA version """
	if not hasattr(self, 'IVA_VERSION'):
		return "IVA"
	else:
		return self.IVA_VERSION
	

    # FIXME: What should we do if self.webtop_quota_amount is invalid?
    security.declareProtected(perm_view, 'get_quota')
    def get_quota(self):
        """Return quota limit in bytes (-1 if no quota in use)."""
        if not hasattr(self, 'webtop_quota'):
          return -1
        if not hasattr(self, 'webtop_quota_amount'):
          return -1
        if not self.webtop_quota:
            return -1 # no quota
        s = self.webtop_quota_amount.strip()
        if s[-1].lower() == 'k':
            return int(s[:-1]) * 1024
        elif s[-1].lower() == 'm':
            return int(s[:-1]) * 1024 * 1024
        else:
            return int(s)

#XXX: security check!
#    security.declareProtected(perm_view, 'get_languages')
    def get_languages(self):
	""" return list of languages available """
	self.lang_list = ['et','en','fi','ru']
	return self.lang_list
            
    def get_default_language(self):
        "Vaikimisi keel"
        if not hasattr(self, 'default_language'):
          return "en"
        return self.default_language    

    def show_controls_for_developers(self, REQUEST):
        """Return boolean depending on whether we are in debug mode
        or not!"""
        return not not self.show_developer_controls

    # This is used by message_dialog.dtml.
    def message_dialog_handler(self, action, REQUEST):
        """Redirects to the given URL, preserving the
        state information in the url."""
        REQUEST.RESPONSE.redirect(action)

    security.declareProtected(perm_view, 'redirect_to_webtop')
    def redirect_to_webtop(self, REQUEST):
        """Redirect to webtop."""
        uname = str(REQUEST.AUTHENTICATED_USER)
        if (not hasattr(self.fle_users.aq_self, uname)):
        	result = self.fle_users.add_user_fle(uname, ('Member',))

	user = self.fle_users.get_user_info(uname)
	if self.getUseFinger():
		user.fingerNames()
	
        self.kontrolli_kursuse_kaust(self, REQUEST)
        getattr(self.fle_users, uname).kysiKirjadeKataloog()        

	user.kontrolliKasutajaKodutood(REQUEST)
	if user.get_first_name()=='' or user.get_last_name()=='':
			return REQUEST.RESPONSE.redirect(self.fle_root().absolute_url()+'/fle_users/' + uname + '/webtop/edit_user_form')
	if hasattr(user, 'perm_must_change'):
		if user.perm_must_change == 1:
	        	return REQUEST.RESPONSE.redirect(self.fle_root().absolute_url()+'/fle_users/' + uname + '/webtop/edit_user_form')
		else:
	        	return REQUEST.RESPONSE.redirect(self.fle_root().absolute_url()+'/organizer_index')
	else:
	        return REQUEST.RESPONSE.redirect(self.fle_root().absolute_url()+'/organizer_index')

    security.declareProtected(perm_manage, 'manage_workspace')
    def manage_workspace(self,REQUEST):
        """This makes sure that ZCatalog doesn't override manage_workspace
        and make it point to index_html, when we actually want manage_main"""
        REQUEST.RESPONSE.redirect('manage_main')

    def logout(self,REQUEST):
        """This is the logout instruction page."""
        return self.message_dialog2(
            self, REQUEST,
            title= 'Logout',
            message="To log out, either close your browser or press the <b>Logout</b> button below. When your browser asks for a username and password, just press <b>Cancel</b>. This will clear your login information from your browser's cache",
            extra_value_name = '',
            extra_values = (),
            option1_value = 'Cancel',
            option1_name = 'cancel',
            option2_value = 'Logout',
            option2_name = 'logout',
            handler = 'clear_password')

    security.declarePublic('clear_password')
    def clear_password(
        self,
        cancel=None,
        logout=None,
        REQUEST=None):
        """This will clear the password cache."""
        if logout:
            REQUEST.RESPONSE.setStatus(401)
            REQUEST.RESPONSE.setHeader("WWW-Authenticate", 'basic realm="Zope"')
	    try:
		    self.login.logout()
	    except AttributeError:
		    pass
            return self.message_dialog(
                self, REQUEST,
                title='Logout',
                message='You have been logged out.',
                action = 'index_html')
        else:
            return self.redirect_to_webtop(REQUEST)

    def has_PIL(self):
        """Has the system PIL (Python Image Library) installed?"""
        return PIL_imported

    def fix_pictures(self):
        """ fix user pictures """
        kasutajad = self.get_users()
        for kasutaja in kasutajad:
                pilt = kasutaja.get_photo()
                kasutaja.set_photo(pilt)
        return "Done"

    def upgrade(self):
	""" creates a role """
	self._addRole('Items')
	return "done"

    security.declareProtected(perm_manage, 'wikiOperation')
    def wikiOperation(self,REQUEST,alg,lopp,omanik):
	from ImportExportIMS import Importer
	abi = Importer('admin',self.fle_root())
	if hasattr(alg, 'WikiStart'):
		kausta_nimi = "WikiStart"
	elif hasattr(alg, 'WikiAvaLeht'):
		kausta_nimi = "WikiAvaLeht"
	if hasattr(lopp, kausta_nimi):
		lopp._delObject(getattr(lopp,kausta_nimi).__name__)
	fold_obj = lopp.lisa_kaust(kausta_nimi)
	fold_obj.set_author(str(REQUEST.AUTHENTICATED_USER))
	fold_obj.wiki_kaust = 1
	abi.set_omanik(fold_obj,omanik)
	for obj in alg.objectValues('ZWiki Page'):
		kopi = alg.manage_copyObjects(obj.__name__)
		tulem = fold_obj.manage_pasteObjects(kopi)
		element = getattr(fold_obj, obj.__name__)
		abi.set_omanik(element, omanik)
		
	return

    def getId(self):
	""" Return object id. Hack XXX """
	return self.id
    

    security.declarePublic('register_form_handler')
    def register_form_handler(self, REQUEST, 
    	regB='', cancelB='', # buttons
	username='', fname='', lname='', email='', preflang='en',optmess='', numbercheck=0):
	""" registers user """
	if regB and self.get_allow_autoregister():
		ok = 1
		inv_val = ''
		if username and fname and lname and email:
			ok = 1
		if not '@' in email and not '.' in email:
			ok = 0
			inv_val+= ' email'
		else:
			#XXX check email!?
			pass
		if not fname:
			ok = 0
			if inv_val: inv_val+=','
			inv_val+=' first name'
		if not lname:
			ok = 0
			if inv_val: inv_val+=','
			inv_val+=' last name'
		if numbercheck != REQUEST.SESSION.get('securenr'):
			ok = 0
			if inv_val: inv_val+=','
			inv_val+= ' number'
		if not username:
			ok = 0
			if inv_val: inv_val+=','
			inv_val+=' username'
		else:
			if self.fle_users.is_valid_uname(username):
				ok = 0
				if inv_val: inv_val+=','
				inv_val+=' username'
				# XXX: maybe we don't want that!
				# XXX: also check in fle_users folder for user record
				# XXX: check for lowlevel acl_users?
		if not ok:
			return self.register_form(username=username, fname=fname, lname=lname, email=email, optmess=optmess,mess='invalid values: '+inv_val)
		#do queueing
		self._queue_user(username=username, fname=fname, lname=lname, email=email, preflang=preflang,optmess=optmess)
		#XXX: subject of email. Translation!
		message = translate(self,"registrant_waiting", target=preflang)
		if self.saadaEmail(email,'useraccount request', """
		Content-Type:text/plain; charset=utf-8


		"""
		+message):
			return translate(self,"You have been registered, check e-mail",target=self.giveLanguage(REQUEST))
		else:
			return translate(self,"You have been registered but some problems occured when sending email.",target=self.giveLanguage(REQUEST))
	elif cancelB:
		return REQUEST.RESPONSE.redirect('index_html')
	raise "IVA Error", "unknown button in register_form_handler"

    def lostPassword(self,REQUEST, send=0,knimi='', give_email=0):
	if give_email:
		if not knimi: return 0
		try:
			kasutaja = self.fle_users.get_user_info(knimi)
		except:
			return 0
		if kasutaja.get_email() != '':
			return 1
		else:
			return 0
		return 0 # should never reach here
	if not hasattr(self.aq_self,'MailHost'):
		return ""
	if send:
		try:
			kasutaja = self.fle_users.get_user_info(knimi)
		except:
			return 0
		if kasutaja.get_email() == '':
			return 0
		sisu = '\nContent-Type:text/plain; charset=utf-8\n\n\n'
		sisu += "\n"+translate(self,'Message from IVA server',target=self.giveLanguage(REQUEST))
		sisu += "\n"+translate(self,'IVA server aadress:',target=self.giveLanguage(REQUEST))+" "+self.fle_root().absolute_url()
		sisu += "\n"+translate(self,'Username:',target=self.giveLanguage(REQUEST))+" "+knimi
		sisu += "\n"+translate(self,'Password:',target=self.giveLanguage(REQUEST))+" "
		try:
			sisu += kasutaja.getPassword()
		except:
			sisu += translate(self,"Error fetching password",target=self.giveLanguage(REQUEST))
		self.saadaEmail(kasutaja.get_email(),'Lost password',sisu, kasutaja.get_email())
	return "Recover password"

    security.declarePublic('randomImage')
    def randomImage(self):
	""" chooses 3/4 images to place on frontpage """
	imgs = self.images.frontpage.objectValues()
	from random import shuffle
	shuffle(imgs)
	if self.get_allow_autoregister(): count=3 
	else: count = 4
	return imgs[0:count]

    security.declareProtected(perm_manage,'manage_iva_setup_pictures')
    def manage_iva_setup_pictures(self, REQUEST, uploadB='', deleteB='',dele='', newimage=''):
	""" upload picture, delete picture """
	imgContainer = getattr(self.images, 'frontpage')
	if deleteB:
		from types import StringType
		if type(dele) == StringType: dele = (dele,)
		for x in dele:
			imgContainer._delObject(x)
				
	if uploadB and newimage:
		import random
		nid = ""
		for x in range(15):
			nr = 95
			while 90<nr<97:
				nr = random.randint(65,122)
			nid += chr(nr)		
		imgContainer.manage_addImage(nid,newimage, '')
	return REQUEST.RESPONSE.redirect('manage_iva_setup')
	    

    security.declareProtected(perm_manage,'manage_iva_setup_handler')
    def manage_iva_setup_handler(self, REQUEST, 
    	saveB='', # save button
	autoregister=0,
	useQuota=0,
	quota=3000000,
	smtp_host='localhost',
	smtp_port=25,
	mailfrom='',
	lang='en',
	def_passwd=0,
	def_not_change=0,
	def_not_aff=0,
	useFinger = 0,
	fingerHost = '',
	fingerPort = 79
	):
	""" saves IVA settings"""
	if saveB:
		self.Autoregister = autoregister
		self.default_language=lang
		self.webtop_quota = int(useQuota)
		self.webtop_quota_amount = quota
		if not hasattr(self,'MailHost'):
			if len(smtp_host)>0 and smtp_port:
				from Products.MailHost import MailHost
				mailhost = MailHost.MailHost()
				mailhost._init(smtp_host, atoi(smtp_port))
				self._setObject('MailHost', mailhost)
		else:
			if len(smtp_host)>0 and smtp_port:
				mailhost = getattr(self, 'MailHost')
				mailhost.smtp_host = smtp_host
				mailhost.smtp_port = smtp_port
		self.MFROM = mailfrom
		self.def_passwd = def_passwd
		self.def_not_change = def_not_change
		self.def_not_aff = def_not_aff
		self.setUseFinger(useFinger)
		self.setFingerHost(fingerHost)
		self.setFingerPort(fingerPort)
		return self.manage_iva_setup(mess='settings saved succesfully!')
	return

    security.declareProtected(perm_manage, 'unqueue_user')
    def unqueue_user(self, uname):
	del self._pending_users[uname]

    security.declarePrivate('_queue_user')
    def _queue_user(self,username='', fname='', lname='', email='', preflang='en',optmess=''):
	# _pending_users
	if not hasattr(self.aq_self, '_pending_users'):
		self._pending_users = PersistentMapping()
	while self._pending_users.has_key(username):
		username+='x'
	self._pending_users[username] = {'fname':fname,'lname':lname,'email':email,'preflang':preflang,'optmess':optmess}

    security.declareProtected(perm_manage,'get_pending_users')
    def get_pending_users(self):
	""" get pending users """
	return self._pending_users.keys()

    security.declareProtected(perm_manage,'get_pending_details')
    def get_pending_details(self, uname):
	""" get details of pending user """
	return self._pending_users[uname]

    security.declarePrivate('make_random_number')
    def make_random_number(self, REQUEST):
	# generates number and stores in in SESSION
	import random
	rnr = int(random.random()*10000000)
	REQUEST.SESSION.set('securenr',rnr)
	return rnr

    def get_allow_autoregister(self):
	return self.Autoregister
    
    security.declarePublic('testtest')
    def generate_picture(self, REQUEST, RESPONSE):
	""" test"""
	srnr = self.make_random_number(REQUEST)
	import PIL
	from PIL import Image, ImageDraw
	import cStringIO
	s = cStringIO.StringIO()
	pilt = Image.new('RGB',(100,30),(255,255,255))
	pil2 = ImageDraw.Draw(pilt)

	tekst = str(srnr)
	pil2.text((10,10),tekst,fill=(30,30,128))
	
	pilt.save(s,'jpeg')
	s.seek(0)
	RESPONSE.setHeader('Content-Type','image/jpeg')
	return s.read()

    def tolgi(self, sona, REQUEST):
        "Tlgib etteantud jupi. Abifunktsioon DTML-ist tlkimisel"
        return translate(self, sona, target=self.giveLanguage(REQUEST))

    security.declareProtected(perm_manage,'exportIMail')
    def exportIMail(self,REQUEST):
	""" export mailmessages """
	from ImportExportIMS import Kirjutus
	from ImportExportIMS import kirjutaYldM
	import tempfile, os
        name = 'ivamessages.zip'
        path = os.path.join(Globals.INSTANCE_HOME,'var')
        fname = os.path.join(path,name)
        #fname = tempfile.mktemp()
        yld = kirjutaYldM()
        yld.kirjutaYldmanifest('innermail','Innermail messages',base='',href='messages.xml')
        yld.lisaFaili(fname)
        k = Kirjutus(self.fle_root(), self.get_id())
        k.exportMail(fname)
        #os.remove(fname)
        return "Done! Zip-file is located at %s" % fname

    security.declareProtected(perm_manage, 'exportServerSettings')
    def exportServerSettings(self, REQUEST):
	""" export server settings and course categories """
	from ImportExportIMS import Kirjutus
	from ImportExportIMS import kirjutaYldM
	import tempfile, os
	name = 'iva_settings.zip'
	path = os.path.join(Globals.INSTANCE_HOME,'var')
	fname = os.path.join(path,name)
	yld = kirjutaYldM()
	yld.kirjutaYldmanifest('IVA_C','IVA course categories',base='',href='course_cat.xml')
	yld.kirjutaYldmanifest('IVA_S','IVA server settings',base='',href='settings.xml')
	yld.kirjutaYldmanifest('IVA_A','IVA affiliations',base='',href='affiliations.xml')
	yld.lisaFaili(fname)
	k = Kirjutus(self.fle_root(), self.get_id())
	k.looKursuseFail(0)
	
	#... do stuff
	k.exportCourseCategories(fname)
	k.exportServerSettings(fname)
	k.exportAffiliations(fname)
	
	file = open(fname,"rb")
	export_data=file.read()
	file.close()
	os.remove(fname)
	REQUEST.RESPONSE.setHeader('Content-disposition','attachment; filename=iva_settings.zip')
	REQUEST.RESPONSE.setHeader('content-type','application/zip')
	return export_data

	return "Done! Zip-file is located at %s" % fname

    security.declareProtected(perm_manage,'exportICAL')
    def exportICAL(self,REQUEST):
	""" export user events """
        from ImportExportIMS import Kirjutus
        from ImportExportIMS import kirjutaYldM
        import tempfile, os
        name = 'iva_events.zip'
        path = os.path.join(Globals.INSTANCE_HOME,'var')
        fname = os.path.join(path,name)
        #fname = tempfile.mktemp()
        yld = kirjutaYldM()
        yld.kirjutaYldmanifest('IVA','IVA user events',base='IVA/',href='imsmanifest.xml')
#       yld.kirjutaYldmanifest()
        yld.lisaFaili(fname)
        k = Kirjutus(self.fle_root(), self.get_id())
        k.looKursuseFail(0)
        k.kirjutaKasutajaSyndmus(self.fle_users.get_users())
        k.pakiKursusFaili(fname)
        #os.remove(fname)
        return "Done! Zip-file is located at %s" % fname

    security.declareProtected(perm_manage,'usercourselistcacheupdate')
    def usercourselistcacheupdate(self):
	""" update user course list cache """
	import time
	for x in self.fle_users.get_users():
		a = time.time()
		x.user_courses(update=1)
		e = time.time()
		print x.get_uname()," kulus ", e-a
	return "Updated"

    security.declareProtected(perm_view,'getUseFinger')
    def getUseFinger(self):
	""" return if user information should be pulled via finger """
	if not hasattr(self, 'useFinger'):
		return 0
	return self.useFinger

    security.declareProtected(perm_manage,'setUseFinger')
    def setUseFinger(self, state):
	""" set if we should use finger """
	self.useFinger = int(state)

    security.declareProtected(perm_view,'getFingerPort')
    def getFingerPort(self):
	""" return finger port """
	if not hasattr(self, 'fingerPort'):
		return "79"
	return self.fingerPort

    security.declareProtected(perm_manage,'setFingerPort')
    def setFingerPort(self, port):
	""" set finger daemon port """
	self.fingerPort = int(port)

    security.declareProtected(perm_view,'getFingerHost')
    def getFingerHost(self):
	""" return finger daemon host """
	if not hasattr(self, 'fingerHost'):
		return ""
	return self.fingerHost

    security.declareProtected(perm_manage,'setFingerHost')
    def setFingerHost(self, host):
	""" set fingerdaemon host """
	self.fingerHost = host

    def getCourseForID(self, REQUEST, course_id, curUser):
	""" return course instance or set current course id to 0 """
	if not hasattr(self.courses.aq_self, course_id):
		curUser.jooksev_kursus = 0
		return REQUEST.RESPONSE.redirect('organizer_index')
	return getattr(self.courses, course_id)
    
    security.declareProtected(perm_manage,'addCookieCrumbler')
    def addCookieCrumbler(self):
	""" xxx """
	if hasattr(self.aq_self, 'login'):
		print "we have cookiecrumbler"
		return
	cc = CookieCrumbler()
	cc.id = 'login'
	self._setObject('login', cc)
	
	#XXX: picture location!!!
	login_form = """
<html>
<script>
<!---
function fookus(){document.post.__iva_name.focus();}
// -->
</script>
<head></head>
<body onLoad=fookus()>
<img src="images/logo_gif"><br />
<dtml-let came_from="REQUEST.get('came_from', 'http://htk.tpu.ee:8080/IVA/IVA')"
action_url="came_from or 'http://htk.tpu.ee:8080/IVA/IVA'">
<form action="&dtml-action_url;" method="POST" name="post">
<dtml-if expr="REQUEST.get('retry', 0)">
<font color="red">Please try again.</font><br />
<a href="IVA/IVA/password_form?knimi=">Forgot Password?</a>
</dtml-if>
<dtml-if came_from>
<input type="hidden" name="came_from" value="&dtml-came_from;">
</dtml-if>
<table>
<tr>
<td align="left" valign="top">
<strong>Name</strong>
</td>
<td align="left" valign="top">
<input type="TEXT" name="__iva_name" size="20"
value="<dtml-var "REQUEST.get('__iva_name', '')">">
</td>
</tr>
<tr>
<td align="left" valign="top">
<strong>Password</strong>
</td>
<td align="left" valign="top">
<input type="PASSWORD" name="__iva_password" size="20">
</td>
</tr>
<tr valign="top" align="left">
<td></td>
<td><input type="checkbox" name="__iva_persistent" value="1" id="cb_remember" />
<label for="cb_remember">Remember my name.</label>
</td></tr>
<tr>
<td align="left" valign="top">
</td>
<td align="left" valign="top">
<input type="submit" name="submit" value=" Login ">
</td>
</tr>
</table>
</form>
</dtml-let>
</body>
</html>
	"""
	self.manage_addDTMLMethod('login_form','CookieCruymblers login form',login_form)
        getattr(self,'login_form').manage_permission('View',['Authenticated', 'Anonymous',], 1)
	cc = getattr(self, 'login')
	cc.auth_cookie = '__iva'
	cc.name_cookie = '__iva_name'
	cc.pw_cookie = '__iva_password'
	cc.persist_cookie = '__iva_persistent'

	return "ok"


Globals.InitializeClass(FLE)


# FIXME: input_checks (some checks done in FLE.__init__)
def manage_addFLE(
    self, _id, title,
    fle_manager='', fle_manager_pwd='',
    fle_manager_first_name='', fle_manager_last_name='',
    acl_users_mode='use_existing', smtp_host='', smtp_port='', import_data=None,
    REQUEST=None, lang='en', quota=3000000, mfrom='iva@htk.tpu.ee',def_passwd=0,
    def_not_change=0,def_not_aff=0,try_it=0):
    """Create a FLE object. (Product Factory Method)."""

    if try_it:
	    if hasattr(self.aq_self, _id):
	    	return 0
	    else:
	    	return 1
    if hasattr(REQUEST,'def_passwd'):
	    def_passwd = REQUEST.def_passwd
    if hasattr(REQUEST,'def_not_change'):
	    def_not_change = REQUEST.def_not_change
    if hasattr(REQUEST,'def_not_aff'):
	    def_not_aff = REQUEST.def_not_aff

    fle = FLE(
        _id, title, fle_manager, fle_manager_pwd,
        fle_manager_first_name, fle_manager_last_name,
        acl_users_mode, smtp_host, smtp_port, import_data, lang, quota, mfrom,
	def_passwd,def_not_change,def_not_aff)
    self._setObject(_id, fle)
    if REQUEST:
        return self.manage_main(self, REQUEST)

manage_addFLEForm = Globals.HTMLFile('ui/IVA_creation_form', globals())

# EOF
